#!/bin/bash
# Test suite to compare C examples with other language bindings
#
# Copyright (C) 2008 Andrew Ross
#
# This file is part of PLplot.
#
# PLplot is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Library Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PLplot is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with PLplot; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

ret=0

# Compare C results with c++, f77, f95, java, octave, python, tcl, perl, 
# ada and ocaml results
for lang in c++ f77 f95 java octave python tcl perl ada ocaml ; do

  # Check which suffix is used for this binding
  case $lang in 
    c++)
      suffix=cxx
      ;;
    f77)
      suffix=f
      ;;
    f95)
      suffix=f95
      ;;
    java)
      suffix=j
      ;;
    octave)
      suffix=o
      ;;
    python)
      suffix=p
      ;;
    tcl)
      suffix=t
      ;;
    perl)
      suffix=pdl
      ;;
    ada)
      suffix=a
      ;;
    ocaml)
      suffix=ocaml
      ;;
  esac

  missing=""
  different=""

  # Assume that if example 1 is missing then the language is
  # not implemented.
  if [ -f x01${suffix}.psc ] ; then
    
  # Standard non-interactive examples
  # Skip example 14 because it requires two output files
  # Skip example 17 because it is interactive
  for index in 01 02 03 04 05 06 07 08 09 10 11 12 13 15 16 18 19 20 \
      21 22 23 24 25 26 27 28 29 30 ; do
    if [ ! -f x${index}c.psc ] ; then
      echo "C example ${index} is missing"
    else
      if [ ! -f x${index}${suffix}.psc ] ; then
        missing="${missing} ${index}"
      else
        /usr/bin/tail --lines=+9 x${index}c.psc > test1.psc
        /usr/bin/tail --lines=+9 x${index}${suffix}.psc > test2.psc
        /usr/bin/diff -q test1.psc test2.psc 2>&1 > /dev/null
        if [ $? != 0 ] ; then
          different="${different} ${index}"
        fi
      fi
    fi
  done
  echo "${lang}"
  echo "  Missing examples   : ${missing}"
  echo "  Differing examples : ${different}"
  if [ "${different}" != "" ] ; then
    ret=1
  fi
  fi 
done

exit ${ret}

